@@ -77,6 +77,14 @@ SolidPattern::SolidPattern(cairo_pattern_t* cobject, bool has_reference)
 : Pattern(cobject, has_reference)
 {
 }
+void
+SolidPattern::get_rgba(double& red, double& green,
+                        double& blue, double& alpha) const
+{
+  // ignore the return value since we know that this is a solid color pattern
+  cairo_pattern_get_rgba(m_cobject, &red, &green, &blue, &alpha);
+  check_object_status_and_throw_exception(*this);
+}
 
 SolidPattern::~SolidPattern()
 {
@@ -103,6 +111,22 @@ SurfacePattern::SurfacePattern(const RefPtr<Surface>& surface)
   check_object_status_and_throw_exception(*this); 
 }
 
+RefPtr<Surface>
+SurfacePattern::get_surface()
+{
+  cairo_surface_t* surface = 0;
+  // we can ignore the return value since we know this is a surface pattern
+  cairo_pattern_get_surface(const_cast<cairo_pattern_t*>(m_cobject), &surface);
+  check_object_status_and_throw_exception(*this);
+  return RefPtr<Surface>(new Surface(surface, false /* does not have reference */));
+}
+
+RefPtr<const Surface>
+SurfacePattern::get_surface() const
+{
+  return const_cast<SurfacePattern*>(this)->get_surface();
+}
+
 RefPtr<SurfacePattern> SurfacePattern::create(const RefPtr<Surface>& surface)
 {
   return RefPtr<SurfacePattern>(new SurfacePattern(surface));
@@ -170,6 +194,26 @@ void Gradient::add_color_stop_rgba(double offset, double red, double green, doub
   check_object_status_and_throw_exception(*this);
 }
 
+std::vector<ColorStop>
+Gradient::get_color_stops() const
+{
+  std::vector<ColorStop> stops;
+
+  int num_stops = 0;
+  // we can ignore the return value since we know this is a gradient pattern
+  cairo_pattern_get_color_stop_count(m_cobject, &num_stops);
+  // since we know the total number of stops, we can avoid re-allocation with
+  // each addition to the vector by pre-allocating the required number
+  stops.reserve(num_stops);
+  for(int i = 0; i < num_stops; ++i)
+  {
+    ColorStop stop;
+    cairo_pattern_get_color_stop_rgba(m_cobject, i, &stop.offset, &stop.red,
+                                      &stop.green, &stop.blue, &stop.alpha);
+    stops.push_back(stop);
+  }
+  return stops;
+}
 
 
 LinearGradient::LinearGradient(double x0, double y0, double x1, double y1)
@@ -178,6 +222,17 @@ LinearGradient::LinearGradient(double x0, double y0, double x1, double y1)
   check_object_status_and_throw_exception(*this); 
 }
 
+void
+LinearGradient::get_linear_points(double &x0, double &y0,
+                                   double &x1, double &y1) const
+{
+  // ignore the return value since we know that this is a linear gradient
+  // pattern
+  cairo_pattern_get_linear_points(m_cobject, &x0, &y0, &x1, &y1);
+  check_object_status_and_throw_exception(*this);
+}
+
+
 RefPtr<LinearGradient> LinearGradient::create(double x0, double y0, double x1, double y1)
 {
   return RefPtr<LinearGradient>(new LinearGradient(x0, y0, x1, y1));
@@ -199,6 +254,18 @@ RadialGradient::RadialGradient(double cx0, double cy0, double radius0, double cx
   check_object_status_and_throw_exception(*this); 
 }
 
+void
+RadialGradient::get_radial_circles(double& x0, double& y0, double& r0,
+                                    double& x1, double& y1, double& r1) const
+{
+  // ignore the return value since we know that this is a radial gradient
+  // pattern
+  cairo_pattern_get_radial_circles(const_cast<cairo_pattern_t*>(m_cobject),
+                                    &x0, &y0, &r0, &x1, &y1, &r1);
+  check_object_status_and_throw_exception(*this); 
+}
+
+
 RefPtr<RadialGradient> RadialGradient::create(double cx0, double cy0, double radius0, double cx1, double cy1, double radius1)
 {
   return RefPtr<RadialGradient>(new RadialGradient(cx0, cy0, radius0, cx1, cy1, radius1));
